'use strict';

exports.__esModule = true;
exports.ResourceRequest = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactRedux = require('react-redux');

var _reduxResource = require('redux-resource');

var _reactRequest = require('react-request');

var _resources = require('./resources');

var _resources2 = _interopRequireDefault(_resources);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class ResourceRequest extends _react.Component {
  render() {
    const {
      request,
      children,
      resourceName,
      transformData,
      crudAction,
      resources,
      dispatch,
      list,
      mergeListIds,
      treatNullAsPending
    } = this.props;

    const capitalizedCrudAction = crudAction.toUpperCase();

    return _react2.default.cloneElement(request, {
      beforeFetch(info) {
        const { requestKey } = info;
        dispatch({
          resourceName,
          resources,
          type: _reduxResource.actionTypes[`${capitalizedCrudAction}_RESOURCES_PENDING`],
          request: requestKey
        });

        request.props.beforeFetch(info);
      },
      afterFetch(info) {
        const { requestKey, data, response, error } = info;
        const status = response && response.status || 0;
        if (error || status >= 400) {
          // This needs to be a more accurate check :)
          if (error && error.isAborted) {
            dispatch({
              resourceName,
              resources,
              type: _reduxResource.actionTypes[`${capitalizedCrudAction}_RESOURCES_NULL`],
              request: requestKey
            });
          } else {
            dispatch({
              resourceName,
              resources,
              statusCode: status,
              type: _reduxResource.actionTypes[`${capitalizedCrudAction}_RESOURCES_FAILED`],
              request: requestKey,
              error
            });
          }
        } else {
          let dispatchResources;
          if (data && transformData) {
            dispatchResources = transformData(data);
          } else if (data) {
            dispatchResources = data;
          } else {
            dispatchResources = resources;
          }
          dispatch({
            resourceName,
            list,
            mergeListIds,
            statusCode: status,
            type: _reduxResource.actionTypes[`${capitalizedCrudAction}_RESOURCES_SUCCEEDED`],
            request: requestKey,
            resources: dispatchResources
          });
        }

        request.props.afterFetch(info);
      },
      children(renderProps) {
        function setFetchToIdle(actionAttributes) {
          return dispatch(_extends({
            type: _reduxResource.actionTypes[`${capitalizedCrudAction}_RESOURCES_NULL`],
            request: renderProps.requestKey,
            resourceName,
            // Should `resources` be in here by default? I need to think more about
            // that.
            resources
          }, actionAttributes));
        }
        return _react2.default.createElement(_resources2.default, _extends({}, renderProps, {
          treatNullAsPending: treatNullAsPending,
          resourceName: resourceName,
          setFetchToIdle: setFetchToIdle,
          children: children,
          list: list
        }));
        return;
      }
    });
  }
}

exports.ResourceRequest = ResourceRequest;
ResourceRequest.propTypes = {
  resourceName: _propTypes2.default.string.isRequired,
  crudAction: _propTypes2.default.string.isRequired,
  lists: _propTypes2.default.arrayOf(_propTypes2.default.string).isRequired
};

ResourceRequest.defaultProps = {
  lists: []
};

function mapDispatchToProps(dispatch) {
  return { dispatch };
}

exports.default = (0, _reactRedux.connect)(null, mapDispatchToProps)(ResourceRequest);